@@ -5,13 +5,11 @@ use crate::utils::GoFloatMath;
 use cryiorust::frame::{Array, FrameError, FrameResult};
 use itertools::izip;
 use std::cmp;
-use std::f64::consts::PI;
+use std::f64::consts::{PI, TAU};
 use std::sync::Arc;
 
 const TTH2Q: f64 = 4e-9 * PI;
 const DSA: i32 = 3;
-const PI_2: f64 = PI * 2.;
-const R2D: f64 = 180.0f64 / PI;
 
 /// Enum with possible integration units.
 #[derive(PartialEq)]
@@ -184,8 +182,8 @@ impl Integrator {
         if self.radial.start < 0. {
             self.radial.start = 0.
         }
-        if self.azimuth.stop > PI_2 {
-            self.azimuth.stop = PI_2
+        if self.azimuth.stop > TAU {
+            self.azimuth.stop = TAU
         }
         if self.azimuth.start < 0. {
             self.azimuth.start = 0.
@@ -281,10 +279,10 @@ impl Positions {
     fn calc_bins(&mut self, units: &Units) {
         self.step = (self.stop - self.start) / self.bins as f64;
         let start = self.start + 0.5 * self.step;
-        let stop = self.stop + 0.5 * self.step;
+        let stop = self.stop - 0.5 * self.step;
         let step = (stop - start) / (self.bins - 1) as f64;
         self.pos = Arc::new(match units {
-            Units::TwoTheta => self.linspace(start * R2D, step * R2D),
+            Units::TwoTheta => self.linspace(start.to_degrees(), step.to_degrees()),
             Units::Qnm | Units::QA => self.linspace(start, step),
         });
     }
@@ -451,7 +449,7 @@ impl<'a> Corners<'a> {
                 self.dtth.push(self.tth2q(dt1.hypot(dt2).atan2(dt3)));
                 let mut chi = dt1.atan2(dt2);
                 if chi < 0. {
-                    chi += PI_2;
+                    chi += TAU;
                 }
                 self.dchi.push(chi);
                 if i != self.params.dim1 && j != self.params.dim2 {
@@ -465,7 +463,7 @@ impl<'a> Corners<'a> {
                     );
                     self.tth.push(self.tth2q(tth));
                     if chi < 0. {
-                        chi += PI_2;
+                        chi += TAU;
                     }
                     self.chi.push(chi);
                 }
@@ -495,7 +493,7 @@ impl<'a> Corners<'a> {
         self.cchi[n].push(
             (*self.dchi.last().unwrap() - self.chi[i])
                 .abs()
-                .remainder(PI_2), // see comments in utils.rs
+                .remainder(TAU), // see comments in utils.rs
         );
         self.counters[n] = i + 1;
     }
@@ -727,59 +725,60 @@ impl Diffractogram {
             buffers.counts[bin.max] += d_ar;
             buffers.values[bin.max] += intensity * d_ar;
             if bin.min + 1 < bin.max {
+                let d_ai = intensity * d_a;
                 for j in (bin.min + 1)..bin.max {
                     buffers.counts[j] += d_a;
-                    buffers.values[j] += intensity * d_a;
+                    buffers.values[j] += d_ai;
                 }
             }
         }
     }
 
-    fn split_pixel_2d(&self, intensity: f64, r: &Bin, a: &Bin, buffers: &mut Buffers) {
-        let i = r.min * a.count;
-        let j = i + a.min;
-        let k = i + a.max;
-        let n = r.max * a.count;
-        let o = n + a.min;
-        let pp = n + a.max;
-        if r.min == r.max {
-            if a.min == a.max {
+    fn split_pixel_2d(&self, intensity: f64, p1: &Bin, p2: &Bin, buffers: &mut Buffers) {
+        let i = p1.min * p2.count;
+        let j = i + p2.min;
+        let k = i + p2.max;
+        let n = p1.max * p2.count;
+        let o = n + p2.min;
+        let pp = n + p2.max;
+        if p1.min == p1.max {
+            if p2.min == p2.max {
                 buffers.counts[j] += 1.;
                 buffers.values[j] += intensity;
             } else {
-                let d_d = (a.min + 1) as f64 - a.min_f;
-                let d_u = a.max_f - a.max as f64;
-                let d_a = (a.max_f - a.min_f).recip();
+                let d_d = (p2.min + 1) as f64 - p2.min_f;
+                let d_u = p2.max_f - p2.max as f64;
+                let d_a = (p2.max_f - p2.min_f).recip();
                 buffers.counts[j] += d_a * d_d;
                 buffers.values[j] += intensity * d_a * d_d;
                 buffers.counts[k] += d_a * d_u;
                 buffers.values[k] += intensity * d_a * d_u;
-                for l in a.min + 1..a.max {
+                for l in p2.min + 1..p2.max {
                     let m = i + l;
                     buffers.counts[m] += d_a;
                     buffers.values[m] += intensity * d_a;
                 }
             }
         } else {
-            if a.min == a.max {
-                let d_a = (r.max_f - r.min_f).recip();
-                let d_l = (r.min + 1) as f64 - r.min_f;
-                let d_r = r.max_f - r.max as f64;
+            if p2.min == p2.max {
+                let d_a = (p1.max_f - p1.min_f).recip();
+                let d_l = (p1.min + 1) as f64 - p1.min_f;
+                let d_r = p1.max_f - p1.max as f64;
                 buffers.counts[j] += d_a * d_l;
                 buffers.values[j] += intensity * d_a * d_l;
                 buffers.counts[o] += d_a * d_r;
                 buffers.values[o] += intensity * d_a * d_r;
-                for l in r.min + 1..r.max {
-                    let m = l * a.count + a.min;
+                for l in p1.min + 1..p1.max {
+                    let m = l * p2.count + p2.min;
                     buffers.counts[m] += d_a;
                     buffers.values[m] += intensity * d_a;
                 }
             } else {
-                let d_l = (r.min + 1) as f64 - r.min_f;
-                let d_r = r.max_f - r.max as f64;
-                let d_d = (a.min + 1) as f64 - a.min_f;
-                let d_u = a.max_f - a.max as f64;
-                let d_a = ((r.max_f - r.min_f) * (a.max_f - a.min_f)).recip();
+                let d_l = (p1.min + 1) as f64 - p1.min_f;
+                let d_r = p1.max_f - p1.max as f64;
+                let d_d = (p2.min + 1) as f64 - p2.min_f;
+                let d_u = p2.max_f - p2.max as f64;
+                let d_a = ((p1.max_f - p1.min_f) * (p2.max_f - p2.min_f)).recip();
                 buffers.counts[j] += d_a * d_l * d_d;
                 buffers.values[j] += intensity * d_a * d_l * d_d;
                 buffers.counts[k] += d_a * d_l * d_u;
@@ -788,20 +787,20 @@ impl Diffractogram {
                 buffers.values[o] += intensity * d_a * d_r * d_d;
                 buffers.counts[pp] += d_a * d_r * d_u;
                 buffers.values[pp] += intensity * d_a * d_r * d_u;
-                for l in r.min + 1..r.max {
-                    let m = l * a.count + a.min;
+                for l in p1.min + 1..p1.max {
+                    let m = l * p2.count + p2.min;
                     buffers.counts[m] += d_a * d_d;
                     buffers.values[m] += intensity * d_a * d_d;
-                    for q in a.min + 1..a.max {
-                        let s = l * a.count + q;
+                    for q in p2.min + 1..p2.max {
+                        let s = l * p2.count + q;
                         buffers.counts[s] += d_a;
                         buffers.values[s] += intensity * d_a;
                     }
-                    let s = l * a.count + a.max;
+                    let s = l * p2.count + p2.max;
                     buffers.counts[s] += d_a * d_u;
                     buffers.values[s] += intensity * d_a * d_u;
                 }
-                for l in a.min + 1..a.max {
+                for l in p2.min + 1..p2.max {
                     let q = i + l;
                     let s = n + l;
                     buffers.counts[q] += d_a * d_l;
@@ -815,15 +814,14 @@ impl Diffractogram {
 
     fn sum_bins(&self, buffers: &mut Buffers) {
         for (counts, value) in buffers.counts.iter_mut().zip(buffers.values.iter_mut()) {
-            if *counts <= 0. {
-                *counts = 0.;
+            if *counts == 0. {
                 *value = 0.;
-                continue;
+            } else {
+                let i = *value / *counts;
+                let e = value.sqrt() / *counts;
+                *counts = i;
+                *value = e;
             }
-            let intensity = (*value / *counts).ok();
-            let sigma = (value.sqrt() / *counts).ok();
-            *counts = intensity;
-            *value = sigma;
         }
     }
 
@@ -862,20 +860,6 @@ impl Diffractogram {
     }
 }
 
-trait Finite {
-    fn ok(self) -> Self;
-}
-
-impl Finite for f64 {
-    fn ok(self) -> Self {
-        if self < 0. || !self.is_finite() {
-            0.
-        } else {
-            self
-        }
-    }
-}
-
 struct Bin<'a> {
     count: usize,
     min: usize,
@@ -911,6 +895,40 @@ impl<'a> Bin<'a> {
     }
 }
 
+impl Cake {
+    /// Fast access to the first radial value if any
+    pub fn radial_first(&self) -> f64 {
+        match self.radial_positions.first() {
+            None => 0.,
+            Some(v) => *v,
+        }
+    }
+
+    /// Fast access to the last radial value if any
+    pub fn radial_last(&self) -> f64 {
+        match self.radial_positions.last() {
+            None => 0.,
+            Some(v) => *v,
+        }
+    }
+
+    /// Fast access to the first azimuthal value if any
+    pub fn azimuthal_first(&self) -> f64 {
+        match self.azimuthal_positions.first() {
+            None => 0.,
+            Some(v) => *v,
+        }
+    }
+
+    /// Fast access to the last azimuthal value if any
+    pub fn azimuthal_last(&self) -> f64 {
+        match self.azimuthal_positions.last() {
+            None => 0.,
+            Some(v) => *v,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::integrator::{Integrable, IntegrationType, Integrator, PatternType, Units};
@@ -1050,6 +1068,32 @@ mod tests {
         let (pos, int, sigma) = get_data("testdata/sample_azimuthal.dat").unwrap();
         compare(&pos, &i.azimuth.pos, "azimuthal angle");
         compare(&int, &p.intensity, "intensity");
-        compare(&sigma, &p.sigma, "intensity");
+        compare(&sigma, &p.sigma, "sigma");
+    }
+
+    #[test]
+    fn test_integrate_cake() {
+        let frame = get_frame_data("testdata/sample_frame.dat");
+        let ranges = [0., 0.];
+        let data = Integrable {
+            array: frame.array(),
+            radial_range: &ranges,
+            azimuthal_range: &ranges,
+            integration_type: IntegrationType::Cake,
+        };
+        let mut i = Integrator::new();
+        i.set_poni(Poni::read_file("testdata/test.poni").unwrap());
+        i.set_polarization(0.99);
+        i.set_radial_bins(360);
+        i.set_azimuthal_bins(360);
+        i.init(frame.array());
+        let p = i.integrate(&data).unwrap();
+        let p = if let PatternType::Cake(p) = p.data {
+            p
+        } else {
+            panic!("Wrong integration result");
+        };
+        let sample = get_frame_data("testdata/sample_cake.dat");
+        compare(&sample.array().data(), &p.cake.data(), "cake");
     }
 }
