@@ -105,7 +105,7 @@ impl Poni {
                     &_ => continue,
                 },
                 PoniValue::Str(v) => match item.key {
-                    "Detector" => detector = v.to_string(),
+                    "Detector" => detector = v.to_lowercase().replace(' ', "_"),
                     &_ => continue,
                 },
                 PoniValue::Pixels(v) => {
@@ -127,16 +127,9 @@ impl Poni {
         if self.pixel1 != 0. && self.pixel2 != 0. {
             return;
         }
-        let pixel = match detector {
-            "Pilatus 2M" | "Pilatus2M" => 172e-6,
-            "FReLoN" => 50e-6,
-            _ => {
-                eprintln!("Unknown detector {}!", detector);
-                0.
-            }
-        };
-        self.pixel1 = pixel;
-        self.pixel2 = pixel;
+        let pixel = detector.to_pixel();
+        self.pixel1 = pixel.0;
+        self.pixel2 = pixel.1;
     }
 }
 
@@ -190,6 +183,75 @@ impl PoniItemParser for str {
     }
 }
 
+/// Tries to guess a pixel size from the detector name
+pub trait ToPixel {
+    fn to_pixel(&self) -> (f64, f64);
+}
+
+impl ToPixel for str {
+    fn to_pixel(&self) -> (f64, f64) {
+        match self {
+            "pilatus100k" | "pilatus_100k" | "pilatus200k" | "pilatus_200k" | "pilatus300k"
+            | "pilatus_300k" | "pilatus300kw" | "pilatus_300kw" | "pilatus1m" | "pilatus_1m"
+            | "pilatus2m" | "pilatus_2m" | "pilatus6m" | "pilatus_6m" | "pilatuscdte300k"
+            | "pilatus_cdte_300k" | "pilatus_300k_cdte" | "pilatus300kcdte"
+            | "pilatus300k_cdte" | "pilatuscdte300kw" | "pilatus_cdte_300kw"
+            | "pilatus_300kw_cdte" | "pilatus300kwcdte" | "pilatus300kw_cdte"
+            | "pilatuscdte900kw" | "pilatus_cdte_900kw" | "pilatus_900kw_cdte"
+            | "pilatus900kwcdte" | "pilatus900kw_cdte" | "pilatuscdte1m" | "pilatus_cdte_1m"
+            | "pilatus_1m_cdte" | "pilatus1mcdte" | "pilatus1m_cdte" | "pilatuscdte2m"
+            | "pilatus_cdte_2m" | "pilatus_2m_cdte" | "pilatus2mcdte" | "pilatus2m_cdte" => {
+                (172e-6, 172e-6)
+            }
+            "eiger500k" | "eiger_500k" | "eiger1m" | "eiger_1m" | "eiger4m" | "eiger_4m"
+            | "eiger9m" | "eiger_9m" | "eiger16m" | "eiger_16m" | "eiger2_500k" | "eiger2500k"
+            | "eiger2_1m" | "eiger21m" | "eiger2_1mw" | "eiger2_1m-w" | "eiger21m-w"
+            | "eiger2_2mw" | "eiger2_2m-w" | "eiger22m-w" | "eiger2_4m" | "eiger24m"
+            | "eiger2_9m" | "eiger29m" | "eiger2_16m" | "eiger216m" | "eiger2cdte_500k"
+            | "eiger2_cdte_500k" | "eiger2cdte500k" | "eiger2cdte_1m" | "eiger2_cdte_1m"
+            | "eiger2cdte1m" | "eiger2cdte_1mw" | "eiger2_cdte_1m-w" | "eiger2cdte1m-w"
+            | "eiger2cdte_2mw" | "eiger2_cdte_2m-w" | "eiger2cdte2m-w" | "eiger2cdte_4m"
+            | "eiger2_cdte_4m" | "eiger2cdte4m" | "eiger2cdte_9m" | "eiger2_cdte_9m"
+            | "eiger2cdte9m" | "eiger2cdte_16m" | "eiger2_cdte_16m" | "eiger2cdte16m"
+            | "jungfrau" | "jungfrau_500k" | "jungfrau500k" | "jungfrau_16m_cor"
+            | "jungfrau16mcor" | "dexela2923" | "dexela_2923" => (75e-6, 75e-6),
+            "frelon" => (50e-6, 50e-6),
+            "maxipix" | "maxipix_1x1" | "maxipix1x1" | "maxipix2x2" | "maxipix_2x2"
+            | "maxipix5x1" | "maxipix_5x1" => (55e-6, 55e-6),
+            "mythen" | "mythen_1280" | "mythen1280" => (8e-3, 50e-6),
+            "adsc_q315" | "quantum_315" | "quantum315" | "adsc_q210" | "quantum_210"
+            | "quantum210" => (51e-6, 51e-6),
+            "adsc_q270" | "quantum_270" | "quantum270" => (64.8e-6, 64.8e-6),
+            "adsc_q4" | "quantum_4" | "quantum4" => (64.8e-6, 64.8e-6),
+            "hf_130k" | "hf-130k" | "hf_262k" | "hf-262k" | "hf_1m" | "hf-1m" | "hf_2m"
+            | "hf-2.4m" | "hf_4m" | "hf-4m" | "hf_9m" | "hf-9.4m" => (150e-6, 150e-6),
+            "imxpads10" | "imxpad_s10" | "imxpads70" | "imxpad_s70" | "imxpads70v"
+            | "imxpad_s70_v" | "imxpads140" | "imxpad_s140" | "xpad_flat" | "xpad_s540_flat"
+            | "xpads540flat" | "d5" | "cirpad" | "xcirpad" => (130e-6, 130e-6),
+            "fairchild" | "condor" | "fairchild_condor_486:90" | "fairchildcondor486:90" => {
+                (15e-6, 15e-6)
+            }
+            "titan" | "titan_2k_x_2k" | "titan2kx2k" | "oxd_titan" | "oxdtitan"
+            | "agilent_titan" | "agilenttitan" => (60e-6, 60e-6),
+            "basler" | "aca1300" => (3.75e-6, 3.75e-6),
+            "perkin" | "perkin_detector" | "perkindetector" | "perkin_elmer" | "perkinelmer" => {
+                (200e-6, 200e-6)
+            }
+            "pixium"
+            | "pixium_4700"
+            | "pixium4700"
+            | "pixium_4700_detector"
+            | "pixium4700detector"
+            | "thales_electronics"
+            | "thaleselectronics" => (308e-6, 308e-6),
+            _ => {
+                eprintln!("Unknown detector name '{}'!", self);
+                (0., 0.)
+            }
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use crate::poni::Poni;
